#include <AlignedJoy.h>
#include <Arduino.h>
#include "Arduino_LED_Matrix.h"
#include "NvmEeprom.hpp"
#include "SingleAxis.hpp"
#include "OneButton.h"
#include "TrainUi.hpp"

// joystick calibration
#define TIME_CAL_1 2000
#define TIME_CAL_2 3000

// static objects
AlignedJoy joystick_1(PIN_A0, PIN_A1);  /// X on A0, Y on A1
ArduinoLEDMatrix ledMatrix;
LinearSingleAxis lever0(PIN_A2);

TrainUi s_ui;

UART *uOut = nullptr;

// enum Button : size_t {
//     RED_BUTTON = 0,
//     // WHITE_BUTTON,

//     NUM_BUTTONS,
// };


// OneButton s_buttons[Button::NUM_BUTTONS] = {
//     [Button::RED_BUTTON] = OneButton(PIN_D2, true, false),
//     // [Button::WHITE_BUTTON] = OneButton(PIN_D3),
// };


static_assert(sizeof(void*) == sizeof(size_t));
void buttonClickHandler(void* ctx) {
    // void* theButton = ctx;

    Serial.println("Clicky clack");

    // char buf[64] = {};
    // snprintf(buf, 64, "Button %u pressed!", theButton);
    // Serial.println(buf);
}

/// Run calibration on the provided joystick. Save the results to the provided calibration
bool runJoystickCalibrationRoutine(AlignedJoy& joy, JoystickCalibration& cal) {
    /*
     * Start the joystick calibration in the center position. Use this method only if the
     * calibration of the axles is desired (axesCalibration).
     */
    // CALIBRATION 1
    Serial.print(
        "   Center calibration \n\n   Leave the joystick in the centre position during "
        "the calibration\n   time which is set to ");
    Serial.print(TIME_CAL_1);
    Serial.println(" milliseconds.");
    delay(5000);
    Serial.println("   Calibration started!");
    delay(500);
    /*
     * for calibrate the joystick center point use "centerCalibration" method;
     * objectname.centerCalibration(uint16_t timeOfCal). "timeOfCal" is the calibration time in
     * milliseconds
     */
    joy.middleCalibration(TIME_CAL_1);
    Serial.println("   Joystick Centered!");
    delay(2000);

    // CALIBRATION 2
    /*
     * Calibration of the axes at the extreme points (min end max for each axis).
     * It is recommended to rotate the joystick in a circular way along its circumference throughout
     * the calibration time.
     */
    Serial.print(
        "   Start axes calibration \n\n   Rotate the joystick in a circular way along its "
        "circumference\n   throughout the calibration time (");
    Serial.print(TIME_CAL_2);
    Serial.println(" milliseconds).");
    delay(5000);
    Serial.println("   Calibration started!");
    delay(500);
    /*
     * for calibrate the joystick axes points use "axesCalibration" method (bool type);
     * objectname.axesCalibration(uint16_t timeOfCal). "timeOfCal" is the calibration time in
     * milliseconds.
     */
    if (joy.axesCalibration(TIME_CAL_2)) {
        Serial.println("   Calibration succesfull!");
    }
    else {
        Serial.println("   Calibration failed!");
        return false;
    }

    // Print all points calibrated
    /*
     * You can use these values to save them to eeprom memory. In this way you will avoid requiring
     * the joystick calibration at each boot time. To set the parameters read by eeprom you have to
     * use the "setCalibratedPoint" method. If your project does not require the re-calibration of
     * the joystick then you can make a sketch like this only to display the calibrated values to
     * set them in the final project using the "setCalibratedPoint" method. To get the calibrated
     * point values use getCalibratedPoint(axis_t axis, point_t point). The parameters: the labels
     * of the "axis_t" shall be X and Y; the labels of "point_t" is MIN, MID and MAX.
     */
    Serial.print("   X min -> ");
    Serial.print(joy.getCalibratedPoint(X, MIN));
    Serial.print(" | center ->  ");
    Serial.print(joy.getCalibratedPoint(X, MID));
    Serial.print(" | max -> ");
    Serial.println(joy.getCalibratedPoint(X, MAX));
    Serial.print("   Y min -> ");
    Serial.print(joy.getCalibratedPoint(Y, MIN));
    Serial.print(" | center ->  ");
    Serial.print(joy.getCalibratedPoint(Y, MID));
    Serial.print(" | max -> ");
    Serial.println(joy.getCalibratedPoint(Y, MAX));

    // save values to cal struct
    cal.x.min = joy.getCalibratedPoint(X, MIN);
    cal.x.mid = joy.getCalibratedPoint(X, MID);
    cal.x.max = joy.getCalibratedPoint(X, MAX);
    cal.y.min = joy.getCalibratedPoint(Y, MIN);
    cal.y.mid = joy.getCalibratedPoint(Y, MID);
    cal.y.max = joy.getCalibratedPoint(Y, MAX);
    return true;
}

bool runLeverCalibration(LinearSingleAxis& lever, LinearAnalogCalibration& cal) {
    Serial.println("   Calibrate lever minimum. Move lever to the min position");
    delay(2000);
    Serial.println("   Capturing min now!");
    cal.min = lever.readRaw();
    delay(500);

    Serial.println("   Calibrate lever maximum. Move lever to the max position");
    delay(2000);
    Serial.println("   Capturing max now!");
    cal.max = lever.readRaw();

    Serial.println("   Calibration succesfull!");
    return true;
}

void setup() {
    // SERIAL INITIALIZE
    Serial.begin(115200);
    ledMatrix.loadSequence(LEDMATRIX_ANIMATION_HEARTBEAT_LINE);
    ledMatrix.begin();
    while (!Serial) {
    }
    ledMatrix.play(true);

    // retrieve current calibration values
    NonVolatileMemory nvm = {};
    readNvm(nvm);
    Serial.println("Wait for serial connection...");
    delay(5000);

    s_ui.setup();

    // run joystick calibration if needed
    // bool calibrationInvalid = !nvm.valid();
    // do {
    //     calibrationInvalid |=
    //         !nvm.calData.joystickCal.x.valid() || !nvm.calData.joystickCal.y.valid();
    //     if (calibrationInvalid) {
    //         Serial.println("Joystick1 calibration required\n========================");
    //         calibrationInvalid =
    //             !runJoystickCalibrationRoutine(joystick_1, nvm.calData.joystickCal);
    //     }
    // } while (calibrationInvalid);
    // // ensure calibration has been applied
    // // TODO: API doesnt allow setting configured midpoint...
    // joystick_1.setCalibratedPoint(axis_t::X, point_t::MIN, nvm.calData.joystickCal.x.min);
    // joystick_1.setCalibratedPoint(axis_t::X, point_t::MID, nvm.calData.joystickCal.x.mid);
    // joystick_1.setCalibratedPoint(axis_t::X, point_t::MAX, nvm.calData.joystickCal.x.max);
    // joystick_1.setCalibratedPoint(axis_t::Y, point_t::MIN, nvm.calData.joystickCal.y.min);
    // joystick_1.setCalibratedPoint(axis_t::Y, point_t::MID, nvm.calData.joystickCal.y.mid);
    // joystick_1.setCalibratedPoint(axis_t::Y, point_t::MAX, nvm.calData.joystickCal.y.max);

    // // run analog lever calibration if needed
    // calibrationInvalid = !nvm.valid();
    // do {
    //     calibrationInvalid |= !nvm.calData.lever0.valid();
    //     if (calibrationInvalid) {
    //         Serial.println("Lever0 calibration required\n===========================");
    //         calibrationInvalid = !runLeverCalibration(lever0, nvm.calData.lever0);
    //     }
    // } while (calibrationInvalid);
    // lever0.applyCalibration(nvm.calData.lever0);

    // // write NVM in the event any values were updated
    // writeNvm(nvm);


    // configure buttons
    // for (size_t ii = 0; ii < NUM_BUTTONS; ii++) {
    //     s_buttons[ii].attachClick(buttonClickHandler, (void*)ii);
    // }

}

int loopCount = 0;

void loop() {
    s_ui.loop();
    delay(10);
    // loopCount++;
    // if (loopCount >= 50) {
    //     loopCount = 0;
    //     // print joystick axes value
    //     Serial.print("joystick_1 X -> ");
    //     Serial.print(joystick_1.read(X));
    //     Serial.print(" | Y -> ");
    //     Serial.println(joystick_1.read(Y));
    //     Serial.print("lever0: ");
    //     Serial.println(lever0.readRaw());
    // }
    // for (size_t ii = 0; ii < NUM_BUTTONS; ii++) {
    //     s_buttons[ii].tick();
    // }
    // delay(10);
}
